import gurobi.*;

public class SPT {
    public static void main(String[] args) throws Exception {
        /*int[][] weights = {
                {Integer.MAX_VALUE, -3, -6, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE},
                {Integer.MAX_VALUE, Integer.MAX_VALUE, -3, Integer.MAX_VALUE, 2, Integer.MAX_VALUE},
                {Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, -4, Integer.MAX_VALUE, 1},
                {Integer.MAX_VALUE, -1, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, 2},
                {Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE},
                {Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE}
        };*/

        int[][] weights = {
                {Integer.MAX_VALUE, 2, 1, 0},
                {Integer.MAX_VALUE, Integer.MAX_VALUE, -4, Integer.MAX_VALUE},
                {1, Integer.MAX_VALUE, Integer.MAX_VALUE, 1},
                {Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE},
        };

        int nSourceNode = 0;
        int nSinkNode = 3;
        int nVertexes = 4;

        // Create new environment.
        GRBEnv env = new GRBEnv();

        // Create empty optimization model.
        GRBModel model = new GRBModel(env);

        // Create variables x.
        // addVar(lowerBound, upperBound, objectiveCoeff, variableType, name)
        GRBVar[][] x = new GRBVar[nVertexes][nVertexes];
        for (int i = 0; i < nVertexes; i++){
            for (int j = 0; j < nVertexes; j++) {
                if(weights[i][j] < Integer.MAX_VALUE){
                    x[i][j] = model.addVar(0.0, 1.0, weights[i][j], GRB.BINARY, "edge" + i + j);
                }
            }
        }

        // Integrate new variables into model.
        model.update();

        // Add constraints:
        // The solution exits source node and enters sink node only once
        GRBLinExpr exitsSourceNode = new GRBLinExpr();
        GRBLinExpr entersSinkNode = new GRBLinExpr();
        for (int j = 0; j < nVertexes; j++) {
            if(weights[nSourceNode][j] < Integer.MAX_VALUE){
                exitsSourceNode.addTerm(1.0, x[nSourceNode][j]);
            }
            if(weights[j][nSinkNode] < Integer.MAX_VALUE){
                entersSinkNode.addTerm(1.0, x[j][nSinkNode]);
            }
        }
        model.addConstr(exitsSourceNode, GRB.EQUAL, 1.0, "exitsSourceNode");
        model.addConstr(entersSinkNode, GRB.EQUAL, 1.0, "entersSinkNode");

        int[] insideVertexes = new int[nVertexes - 2];
        for (int i = 0, j = 0; i < nVertexes; i++) {
            if(i != nSinkNode && i != nSourceNode){
                insideVertexes[j] = i;
                j++;
            }
        }
        
        for (int i : insideVertexes){
            GRBLinExpr exitsEqualEnters = new GRBLinExpr();
            GRBLinExpr exitsAtMostOnce = new GRBLinExpr();
            for (int j = 0; j < nVertexes; j++) {
                if(weights[i][j] < Integer.MAX_VALUE){
                    exitsEqualEnters.addTerm(1.0, x[i][j]);
                    exitsAtMostOnce.addTerm(1.0, x[i][j]);
                }
                if(weights[j][i] < Integer.MAX_VALUE){
                    exitsEqualEnters.addTerm(-1.0, x[j][i]);
                }
            }
            model.addConstr(exitsEqualEnters, GRB.EQUAL, 0.0, "exitsEqualEnters");
            model.addConstr(exitsAtMostOnce, GRB.LESS_EQUAL, 1.0, "entersSinkNode");
        }


        // Solve the model.
        model.optimize();

        // Print the objective
        // and the values of the decision variables in the solution.
        for (int i = 0; i < nVertexes; i++){
            for (int j = 0; j < nVertexes; j++) {
                if(weights[i][j] < Integer.MAX_VALUE && x[i][j].get(GRB.DoubleAttr.X) > 0.5) {
                    System.out.println(i +", " + j );
                }
            }
        }
        System.out.println("shortest s-t distance ILP:" + model.get(GRB.DoubleAttr.ObjVal));

        // bellman-ford
        int[] l = new int[nVertexes];
        for (int i = 0; i < nVertexes; i++) {
            l[i] = Integer.MAX_VALUE;
        }
        l[nSourceNode] = 0;

        for (int i = 0; i < nVertexes - 1; i++) {
            for (int j = 0; j < nVertexes; j++) {
                for (int k = 0; k < nVertexes; k++) {
                    if(weights[j][k] < Integer.MAX_VALUE){
                        if(l[k] > l[j] + weights[j][k]){
                            l[k] = l[j] + weights[j][k];
                        }
                    }
                }
            }
        }

        System.out.println("shortest s-t distance Bellman-Ford:" + l[nSinkNode]);


    }
}
